@@ -18,11 +18,20 @@
 
 #include <mpd/client.h>
 
-#include <stddef.h>                     // for NULL
+#include <stddef.h>
 #include <unistd.h>
-#include <iostream>                     // for endl, etc
+#include <iostream>
+#include <sstream>
+#include <cstdio>
+#include <string>
+#include <memory>
 
-#include "libupnpp/log.hxx"             // for LOGDEB, LOGERR, LOGINF
+#include "libupnpp/log.hxx"
+
+#include "upmpd.hxx"
+#include "conftree.hxx"
+#include "execmd.h"
+#include "upmpdutils.hxx"
 
 struct mpd_status;
 
@@ -33,8 +42,9 @@ using namespace UPnPP;
 
 MPDCli::MPDCli(const string& host, int port, const string& pass)
     : m_conn(0), m_ok(false), m_premutevolume(0), m_cachedvolume(50),
-      m_host(host), m_port(port), m_password(pass), m_lastinsertid(-1),
-      m_lastinsertpos(-1), m_lastinsertqvers(-1)
+      m_host(host), m_port(port), m_password(pass),
+      m_externalvolumecontrol(false),
+      m_lastinsertid(-1), m_lastinsertpos(-1), m_lastinsertqvers(-1)
 {
     regcomp(&m_tpuexpr, "^[[:alpha:]]+://.+", REG_EXTENDED|REG_NOSUB);
     if (!openconn()) {
@@ -44,6 +54,22 @@ MPDCli::MPDCli(const string& host, int port, const string& pass)
 
     m_ok = true;
     m_ok = updStatus();
+
+    UPnPP::PTMutexLocker conflock(g_configlock);
+    g_config->get("onstart", m_onstart);
+    g_config->get("onplay", m_onplay);
+    g_config->get("onstop", m_onstop);
+    string scratch;
+    g_config->get("onvolumechange", scratch);
+    stringToStrings(scratch,  m_onvolumechange);
+    g_config->get("getexternalvolume", scratch);
+    stringToStrings(scratch, m_getexternalvolume);
+    
+    m_externalvolumecontrol = false;
+    string value;
+    if (g_config->get("externalvolumecontrol", value)) {
+        m_externalvolumecontrol = atoi(value.c_str()) != 0;
+    }
 }
 
 MPDCli::~MPDCli()
@@ -53,6 +79,17 @@ MPDCli::~MPDCli()
     regfree(&m_tpuexpr);
 }
 
+// This is used on the auxiliary songcast mpd in a configuration where
+// volume is normally controlled by an external script, but we still
+// want to scale the Songcast stream.
+void MPDCli::forceInternalVControl()
+{
+    m_getexternalvolume.clear();
+    if (m_externalvolumecontrol)
+        m_onvolumechange.clear();
+    m_externalvolumecontrol = false;
+}
+
 bool MPDCli::looksLikeTransportURI(const string& path)
 {
     return (regexec(&m_tpuexpr, path.c_str(), 0, 0, 0) == 0);
@@ -93,7 +130,7 @@ bool MPDCli::showError(const string& who)
 
     int error = mpd_connection_get_error(M_CONN);
     if (error == MPD_ERROR_SUCCESS) {
-        //LOGDEB("MPDCli::showError: " << who << " success !" << endl;);
+        //LOGDEB("MPDCli::showError: " << who << " success !" << endl);
         return false;
     }
     LOGERR(who << " failed: " <<  mpd_connection_get_error_message(M_CONN) 
@@ -147,7 +184,18 @@ bool MPDCli::updStatus()
         return false;
     }
 
-    m_stat.volume = mpd_status_get_volume(mpds);
+    if (m_externalvolumecontrol && !m_getexternalvolume.empty()) {
+        string result;
+        if (ExecCmd::backtick(m_getexternalvolume, result)) {
+            //LOGDEB("MPDCli::volume retrieved: " << result << endl);
+            m_stat.volume = atoi(result.c_str());
+        } else {
+            LOGERR("MPDCli::updStatus: error retrieving volume: " <<
+                   m_getexternalvolume[0] << " failed\n");
+        }
+    } else {
+	m_stat.volume = mpd_status_get_volume(mpds);
+    }
     if (m_stat.volume >= 0) {
         m_cachedvolume = m_stat.volume;
     } else {
@@ -162,8 +210,25 @@ bool MPDCli::updStatus()
     m_stat.qvers = mpd_status_get_queue_version(mpds);
 
     switch (mpd_status_get_state(mpds)) {
-    case MPD_STATE_STOP: m_stat.state = MpdStatus::MPDS_STOP;break;
-    case MPD_STATE_PLAY: m_stat.state = MpdStatus::MPDS_PLAY;break;
+    case MPD_STATE_STOP:
+        // Only execute onstop command if mpd was playing or paused
+        if (!m_onstop.empty() && (m_stat.state == MpdStatus::MPDS_PLAY ||
+                                  m_stat.state == MpdStatus::MPDS_PAUSE)) {
+            if (system(m_onstop.c_str())) {
+                LOGERR("MPDCli::updStatus: " << m_onstop << " failed "<< endl);
+            }
+        }
+        m_stat.state = MpdStatus::MPDS_STOP;
+        break;
+    case MPD_STATE_PLAY:
+        // Only execute onplay command if mpd was stopped
+        if (!m_onplay.empty() && m_stat.state == MpdStatus::MPDS_STOP) {
+            if (system(m_onplay.c_str())) {
+                LOGERR("MPDCli::updStatus: " << m_onplay << " failed "<< endl);
+            }
+        }
+        m_stat.state = MpdStatus::MPDS_PLAY;
+        break;
     case MPD_STATE_PAUSE: m_stat.state = MpdStatus::MPDS_PAUSE;break;
     case MPD_STATE_UNKNOWN: 
     default:
@@ -233,9 +298,63 @@ bool MPDCli::checkForCommand(const string& cmdname)
     return found;
 }
 
+bool MPDCli::saveState(MpdState& st, int seekms)
+{
+    LOGDEB("MPDCli::saveState: seekms " << seekms << endl);
+    if (!updStatus()) {
+        LOGERR("MPDCli::saveState: can't retrieve current status\n");
+        return false;
+    }
+    st.status = m_stat;
+    if (seekms > 0) {
+        st.status.songelapsedms = seekms;
+    }
+    st.queue.clear();
+    if (!getQueueData(st.queue)) {
+        LOGERR("MPDCli::saveState: can't retrieve current playlist\n");
+        return false;
+    }
+    return true;
+}
+
+bool MPDCli::restoreState(const MpdState& st)
+{
+    LOGDEB("MPDCli::restoreState: seekms " << st.status.songelapsedms << endl);
+    clearQueue();
+    for (unsigned int i = 0; i < st.queue.size(); i++) {
+        if (insert(st.queue[i].uri, i, st.queue[i]) < 0) {
+            LOGERR("MPDCli::restoreState: insert failed\n");
+            return false;
+        }
+    }
+    repeat(st.status.rept);
+    random(st.status.random);
+    single(st.status.single);
+    consume(st.status.consume);
+    m_cachedvolume = st.status.volume;
+    //no need to set volume if it is controlled external
+    if (!m_externalvolumecontrol)
+        mpd_run_set_volume(M_CONN, st.status.volume);
+
+    if (st.status.state == MpdStatus::MPDS_PAUSE ||
+        st.status.state == MpdStatus::MPDS_PLAY) {
+        // I think that the play is necessary and we can't just do
+        // pause/seek from stop state. To be verified.
+        play(st.status.songpos);
+        if (st.status.songelapsedms > 0)
+            seek(st.status.songelapsedms/1000);
+        if (st.status.state == MpdStatus::MPDS_PAUSE)
+            pause(true);
+        if (!m_externalvolumecontrol)
+            mpd_run_set_volume(M_CONN, st.status.volume);
+    }
+    return true;
+}
+
+
 bool MPDCli::statSong(UpSong& upsong, int pos, bool isid)
 {
-    //LOGDEB("MPDCli::statSong. isid " << isid << " val " << pos << endl);
+    //LOGDEB1("MPDCli::statSong. isid " << isid << " id/pos " << pos << endl);
     if (!ok())
         return false;
 
@@ -261,6 +380,7 @@ bool MPDCli::statSong(UpSong& upsong, int pos, bool isid)
 
 UpSong&  MPDCli::mapSong(UpSong& upsong, struct mpd_song *song)
 {
+    //LOGDEB1("MPDCli::mapSong" << endl);
     const char *cp;
 
     cp = mpd_song_get_uri(song);
@@ -278,11 +398,18 @@ UpSong&  MPDCli::mapSong(UpSong& upsong, struct mpd_song *song)
         // " replacing [" << upsong.uri << "]" << endl);
         upsong.uri = "http://127.0.0.1/" + upsong.uri;
     }
+    cp = mpd_song_get_tag(song, MPD_TAG_NAME, 0);
+    if (cp != 0)
+        upsong.name = cp;
+    else
+        upsong.name.clear();
     cp = mpd_song_get_tag(song, MPD_TAG_ARTIST, 0);
     if (cp != 0)
         upsong.artist = cp;
     else
         upsong.artist.clear();
+    if (upsong.artist.empty())
+        upsong.artist = upsong.name;
     cp = mpd_song_get_tag(song, MPD_TAG_ALBUM, 0);
     if (cp != 0)
         upsong.album = cp;
@@ -307,20 +434,23 @@ UpSong&  MPDCli::mapSong(UpSong& upsong, struct mpd_song *song)
     upsong.duration_secs = mpd_song_get_duration(song);
     upsong.mpdid = mpd_song_get_id(song);
 
+//    LOGDEB1("MPDCli::mapSong: got mpdid " << upsong.mpdid << " " << 
+//            upsong.dump() << endl);
     return upsong;
 }
 
 bool MPDCli::setVolume(int volume, bool isMute)
 {
+    LOGDEB("MPDCli::setVolume. extvc " << m_externalvolumecontrol << endl);
     if (!ok()) {
         return false;
     }
 
-    // MPD does not want to set the volume if not active.
+    // ??MPD does not want to set the volume if not active.??
+    // This does not seem to be the case with recent MPD versions
     if (!(m_stat.state == MpdStatus::MPDS_PLAY) &&
         !(m_stat.state == MpdStatus::MPDS_PAUSE)) {
-        LOGINF("MPDCli::setVolume: not active" << endl);
-        return true;
+        LOGDEB1("MPDCli::setVolume: not active" << endl);
     }
 
     LOGDEB("MPDCli::setVolume: vol " << volume << " isMute " << isMute << endl);
@@ -350,7 +480,21 @@ bool MPDCli::setVolume(int volume, bool isMute)
     else if (volume > 100)
         volume = 100;
     
-    RETRY_CMD(mpd_run_set_volume(M_CONN, volume));
+    if (!(m_externalvolumecontrol)) {
+    	RETRY_CMD(mpd_run_set_volume(M_CONN, volume));
+    }
+    if (!m_onvolumechange.empty()) {
+        ExecCmd ecmd;
+        vector<string> args = m_onvolumechange;
+        stringstream ss;
+        ss << volume;
+        args.push_back(ss.str());
+        if (ecmd.doexec1(args)) {
+            // doexec returns the exit status, should be zero
+            LOGDEB("MPDCli::setVolume: " << m_onvolumechange[0] <<
+                   " failed " << volume << endl);
+        }
+    }
     m_stat.volume = volume;
     m_cachedvolume = volume;
     return true;
@@ -358,6 +502,7 @@ bool MPDCli::setVolume(int volume, bool isMute)
 
 int MPDCli::getVolume()
 {
+    //LOGDEB1("MPDCli::getVolume" << endl);
     return m_stat.volume >= 0 ? m_stat.volume : m_cachedvolume;
 }
 
@@ -384,6 +529,11 @@ bool MPDCli::play(int pos)
     LOGDEB("MPDCli::play(pos=" << pos << ")" << endl);
     if (!ok())
         return false;
+    if (!m_onstart.empty()) {
+        if (system(m_onstart.c_str())) {
+            LOGERR("MPDCli::play: " << m_onstart << " failed "<< endl);
+        }
+    }
     if (pos >= 0) {
         RETRY_CMD(mpd_run_play_pos(M_CONN, (unsigned int)pos));
     } else {
@@ -397,6 +547,11 @@ bool MPDCli::playId(int id)
     LOGDEB("MPDCli::playId(id=" << id << ")" << endl);
     if (!ok())
         return false;
+    if (!m_onstart.empty()) {
+        if (system(m_onstart.c_str())) {
+            LOGERR("MPDCli::playId: " << m_onstart << " failed "<< endl);
+        }
+    }
     RETRY_CMD(mpd_run_play_id(M_CONN, (unsigned int)id));
     return updStatus();
 }
@@ -477,13 +632,19 @@ bool MPDCli::send_tag(const char *cid, int tag, const string& data)
         return false;
     }
 
-    return mpd_response_finish(M_CONN);
+    if (!mpd_response_finish(M_CONN)) {
+        LOGERR("MPDCli::send_tag: mpd_response_finish failed\n");
+        showError("MPDCli::send_tag");
+        return false;
+    }
+    return true;
 }
 
 static const string upmpdcli_comment("client=upmpdcli;");
 
 bool MPDCli::send_tag_data(int id, const UpSong& meta)
 {
+    LOGDEB1("MPDCli::send_tag_data" << endl);
     if (!m_have_addtagid)
         return false;
 
@@ -606,6 +767,7 @@ bool MPDCli::statId(int id)
 
 bool MPDCli::getQueueSongs(vector<mpd_song*>& songs)
 {
+    //LOGDEB1("MPDCli::getQueueSongs" << endl);
     songs.clear();
 
     RETRY_CMD(mpd_send_list_queue_meta(M_CONN));
@@ -625,6 +787,7 @@ bool MPDCli::getQueueSongs(vector<mpd_song*>& songs)
 
 void MPDCli::freeSongs(vector<mpd_song*>& songs)
 {
+    LOGDEB1("MPDCli::freeSongs" << endl);
     for (vector<mpd_song*>::iterator it = songs.begin();
          it != songs.end(); it++) {
         mpd_song_free(*it);
@@ -633,7 +796,7 @@ void MPDCli::freeSongs(vector<mpd_song*>& songs)
 
 bool MPDCli::getQueueData(std::vector<UpSong>& vdata)
 {
-    //LOGDEB("MPDCli::getQueueData" << endl);
+    LOGDEB("MPDCli::getQueueData" << endl);
     vector<mpd_song*> songs;
     if (!getQueueSongs(songs)) {
         return false;
@@ -658,7 +821,6 @@ int MPDCli::curpos()
 
 
 
-
 #ifdef MPDCLI_TEST
 
 #include <stdio.h>
