@@ -36,6 +36,7 @@
 #include "mpdcli.hxx"                   // for MpdStatus, UpSong, MPDCli, etc
 #include "upmpd.hxx"                    // for UpMpd, etc
 #include "upmpdutils.hxx"               // for didlmake, diffmaps, etc
+#include "ohproduct.hxx"
 
 using namespace std;
 using namespace std::placeholders;
@@ -43,9 +44,10 @@ using namespace std::placeholders;
 static const string sTpProduct("urn:av-openhome-org:service:Playlist:1");
 static const string sIdProduct("urn:av-openhome-org:serviceId:Playlist");
 
-OHPlaylist::OHPlaylist(UpMpd *dev, UpMpdRenderCtl *ctl, unsigned int cssleep)
-    : UpnpService(sTpProduct, sIdProduct, dev), m_dev(dev),
-      m_cachedirty(false), m_mpdqvers(-1)
+// Playlist is the default oh service, so it's active when starting up
+OHPlaylist::OHPlaylist(UpMpd *dev, unsigned int cssleep)
+    : OHService(sTpProduct, sIdProduct, dev),
+      m_active(true), m_cachedirty(false), m_mpdqvers(-1)
 {
     dev->addActionMapping(this, "Play", 
                           bind(&OHPlaylist::play, this, _1, _2));
@@ -95,7 +97,6 @@ OHPlaylist::OHPlaylist(UpMpd *dev, UpMpdRenderCtl *ctl, unsigned int cssleep)
                           bind(&OHPlaylist::idArrayChanged, this, _1, _2));
     dev->addActionMapping(this, "ProtocolInfo",
                           bind(&OHPlaylist::protocolInfo, this, _1, _2));
-    dev->m_mpdcli->consume(false);
     
     if ((dev->m_options & UpMpd::upmpdOhMetaPersist)) {
         dmcacheSetOpts(cssleep);
@@ -141,12 +142,13 @@ static string translateIdArray(const vector<UpSong>& in)
         }
         sdeb += SoapHelp::i2s(val) + " ";
     }
-    LOGDEB("OHPlaylist: current ids: " << sdeb << endl);
+    LOGDEB("OHPlaylist::translateIdArray: current ids: " << sdeb << endl);
     return base64_encode(out1);
 }
 
 bool OHPlaylist::makeIdArray(string& out)
 {
+    //LOGDEB1("OHPlaylist::makeIdArray\n");
     const MpdStatus &mpds = m_dev->getMpdStatusNoUpdate();
 
     if (mpds.qvers == m_mpdqvers) {
@@ -180,11 +182,18 @@ bool OHPlaylist::makeIdArray(string& out)
     m_idArrayCached = out = translateIdArray(vdata);
     m_mpdqvers = mpds.qvers;
 
-    // Update metadata cache: entries not in the current list are
-    // not valid any more. Also there may be entries which were added
+    // Don't perform metadata cache maintenance if we're not active
+    // (the mpd playlist belongs to e.g. the radio service). We would
+    // be destroying data which we may need later.
+    if (!m_active) {
+        return true;
+    }
+
+    // Update metadata cache: entries not in the current list are not
+    // valid any more. Also there may be entries which were added
     // through an MPD client and which don't know about, record the
-    // metadata for these. We don't update the current array, but
-    // just build a new cache for data about current entries.
+    // metadata for these. We don't update the current array, but just
+    // build a new cache for data about current entries.
     //
     // The songids are not preserved through mpd restarts (they
     // restart at 0) this means that the ids are not a good cache key,
@@ -247,29 +256,11 @@ bool OHPlaylist::makestate(unordered_map<string, string> &st)
     return true;
 }
 
-bool OHPlaylist::getEventData(bool all, std::vector<std::string>& names, 
-                              std::vector<std::string>& values)
+void OHPlaylist::refreshState()
 {
-    //LOGDEB("OHPlaylist::getEventData" << endl);
-
-    unordered_map<string, string> state;
-
-    makestate(state);
-
-    unordered_map<string, string> changed;
-    if (all) {
-        changed = state;
-    } else {
-        changed = diffmaps(m_state, state);
-    }
-    m_state = state;
-
-    for (auto it = changed.begin(); it != changed.end(); it++) {
-        names.push_back(it->first);
-        values.push_back(it->second);
-    }
-
-    return true;
+    m_mpdqvers = -1;
+    unordered_map<string, string> st;
+    makestate(st);
 }
 
 void OHPlaylist::maybeWakeUp(bool ok)
@@ -278,9 +269,28 @@ void OHPlaylist::maybeWakeUp(bool ok)
         m_dev->loopWakeup();
 }
 
+void OHPlaylist::setActive(bool onoff)
+{
+    m_active = onoff;
+    if (m_active) {
+        m_dev->m_mpdcli->clearQueue();
+        m_dev->m_mpdcli->restoreState(m_mpdsavedstate);
+        refreshState();
+        maybeWakeUp(true);
+    } else {
+        m_dev->m_mpdcli->saveState(m_mpdsavedstate);
+        iStop();
+    }
+}
+
 int OHPlaylist::play(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::play" << endl);
+    if (!m_active && m_dev->m_ohpr) {
+        m_dev->m_ohpr->iSetSourceIndexByName("Playlist");
+    }
+    m_dev->m_mpdcli->consume(false);
+    m_dev->m_mpdcli->single(false);
     bool ok = m_dev->m_mpdcli->play();
     maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
@@ -294,13 +304,17 @@ int OHPlaylist::pause(const SoapIncoming& sc, SoapOutgoing& data)
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
-int OHPlaylist::stop(const SoapIncoming& sc, SoapOutgoing& data)
+int OHPlaylist::iStop()
 {
-    LOGDEB("OHPlaylist::stop" << endl);
     bool ok = m_dev->m_mpdcli->stop();
     maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
+int OHPlaylist::stop(const SoapIncoming& sc, SoapOutgoing& data)
+{
+    LOGDEB("OHPlaylist::stop" << endl);
+    return iStop();
+}
 
 int OHPlaylist::next(const SoapIncoming& sc, SoapOutgoing& data)
 {
@@ -333,7 +347,7 @@ int OHPlaylist::setRepeat(const SoapIncoming& sc, SoapOutgoing& data)
 int OHPlaylist::repeat(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::repeat" << endl);
-    const MpdStatus &mpds =  m_dev->getMpdStatus();
+    const MpdStatus &mpds =  m_dev->getMpdStatusNoUpdate();
     data.addarg("Value", mpds.rept? "1" : "0");
     return UPNP_E_SUCCESS;
 }
@@ -355,7 +369,7 @@ int OHPlaylist::setShuffle(const SoapIncoming& sc, SoapOutgoing& data)
 int OHPlaylist::shuffle(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::shuffle" << endl);
-    const MpdStatus &mpds =  m_dev->getMpdStatus();
+    const MpdStatus &mpds =  m_dev->getMpdStatusNoUpdate();
     data.addarg("Value", mpds.random ? "1" : "0");
     return UPNP_E_SUCCESS;
 }
@@ -407,7 +421,7 @@ int OHPlaylist::transportState(const SoapIncoming& sc, SoapOutgoing& data)
     default:
         tstate = "Stopped";
     }
-    data.addarg("TransportState", tstate);
+    data.addarg("Value", tstate);
     return UPNP_E_SUCCESS;
 }
 
@@ -415,6 +429,15 @@ int OHPlaylist::transportState(const SoapIncoming& sc, SoapOutgoing& data)
 int OHPlaylist::seekId(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::seekId" << endl);
+    if (!m_active) {
+        // If I'm not active, the ids in the playlist are those of
+        // another service (e.g. radio). If I activate myself and
+        // restore the playlist, the mpd ids are going to be different
+        // from what the caller may have in store. This just can't
+        // work as long as we use mpd ids directly.
+        LOGERR("OHPlaylist::seekId: not active" << endl);
+        return UPNP_E_INTERNAL_ERROR;
+    }
     int id;
     bool ok = sc.get("Value", &id);
     if (ok) {
@@ -428,6 +451,12 @@ int OHPlaylist::seekId(const SoapIncoming& sc, SoapOutgoing& data)
 int OHPlaylist::seekIndex(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::seekIndex" << endl);
+
+    // Unlike seekid, this should work as the indices are restored by
+    // mpdcli restorestate
+    if (!m_active && m_dev->m_ohpr) {
+        m_dev->m_ohpr->iSetSourceIndexByName("Playlist");
+    }
     int pos;
     bool ok = sc.get("Value", &pos);
     if (ok) {
@@ -441,6 +470,11 @@ int OHPlaylist::seekIndex(const SoapIncoming& sc, SoapOutgoing& data)
 int OHPlaylist::id(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::id" << endl);
+    if (!m_active) {
+        LOGERR("OHPlaylist::id: not active" << endl);
+        return UPNP_E_INTERNAL_ERROR;
+    }
+
     const MpdStatus &mpds = m_dev->getMpdStatusNoUpdate();
     data.addarg("Value", mpds.songid == -1 ? "0" : SoapHelp::i2s(mpds.songid));
     return UPNP_E_SUCCESS;
@@ -460,6 +494,11 @@ bool OHPlaylist::cacheFind(const string& uri, string& meta)
 // Returns a 800 fault code if the given id is not in the playlist. 
 int OHPlaylist::ohread(const SoapIncoming& sc, SoapOutgoing& data)
 {
+    if (!m_active) {
+        // See comment in seekId()
+        LOGERR("OHPlaylist::read: not active" << endl);
+        return UPNP_E_INTERNAL_ERROR;
+    }
     int id;
     bool ok = sc.get("Id", &id);
     LOGDEB("OHPlaylist::ohread id " << id << endl);
@@ -497,6 +536,11 @@ int OHPlaylist::ohread(const SoapIncoming& sc, SoapOutgoing& data)
 // Any ids not in the playlist are ignored. 
 int OHPlaylist::readList(const SoapIncoming& sc, SoapOutgoing& data)
 {
+    if (!m_active) {
+        // See comment in seekId()
+        LOGERR("OHPlaylist::readList: not active" << endl);
+        return UPNP_E_INTERNAL_ERROR;
+    }
     string sids;
     bool ok = sc.get("IdList", &sids);
     LOGDEB("OHPlaylist::readList: [" << sids << "]" << endl);
@@ -575,6 +619,20 @@ int OHPlaylist::insert(const SoapIncoming& sc, SoapOutgoing& data)
     if (ok)
         ok = ok && sc.get("Metadata", &metadata);
 
+    if (!m_active) {
+        // See comment in seekId()
+        // It's not clear if special-casing afterId == 0 is a good
+        // idea because it makes the device appear even more
+        // unpredictable. Otoh, it allows Bubble (basic, not DS) to
+        // switch to playlist by just adding tracks.
+        if (afterid == 0 && m_dev->m_ohpr) {
+            m_dev->m_ohpr->iSetSourceIndexByName("Playlist");
+        } else {
+            LOGERR("OHPlaylist::insert: not active" << endl);
+            return UPNP_E_INTERNAL_ERROR;
+        }
+    }
+
     LOGDEB("OHPlaylist::insert: afterid " << afterid << " Uri " <<
            uri << " Metadata " << metadata << endl);
     if (ok) {
@@ -592,6 +650,11 @@ int OHPlaylist::insert(const SoapIncoming& sc, SoapOutgoing& data)
 bool OHPlaylist::insertUri(int afterid, const string& uri, 
                            const string& metadata, int *newid)
 {
+    LOGDEB1("OHPlaylist::insertUri: " << uri << endl);
+    if (!m_active) {
+        LOGERR("OHPlaylist::insertUri: not active" << endl);
+        return UPNP_E_INTERNAL_ERROR;
+    }
     UpSong metaformpd;
     if (!uMetaToUpSong(metadata, &metaformpd)) {
         LOGERR("OHPlaylist::insert: failed to parse metadata " << " Uri [" 
@@ -607,13 +670,18 @@ bool OHPlaylist::insertUri(int afterid, const string& uri,
             *newid = id;
         return true;
     } 
-    LOGERR("OHPlaylist::insert: mpd error" << endl);
+    LOGERR("OHPlaylist::insertUri: mpd error" << endl);
     return false;
 }
 
 int OHPlaylist::deleteId(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::deleteId" << endl);
+    if (!m_active) {
+        // See comment in seekId()
+        LOGERR("OHPlaylist::deleteId: not active" << endl);
+        return UPNP_E_INTERNAL_ERROR;
+    }
     int id;
     bool ok = sc.get("Value", &id);
     if (ok) {
@@ -633,6 +701,9 @@ int OHPlaylist::deleteId(const SoapIncoming& sc, SoapOutgoing& data)
 int OHPlaylist::deleteAll(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::deleteAll" << endl);
+    if (!m_active && m_dev->m_ohpr) {
+        m_dev->m_ohpr->iSetSourceIndexByName("Playlist");
+    }
     bool ok = m_dev->m_mpdcli->clearQueue();
     m_mpdqvers = -1;
     maybeWakeUp(ok);
@@ -651,6 +722,11 @@ int OHPlaylist::tracksMax(const SoapIncoming& sc, SoapOutgoing& data)
 int OHPlaylist::idArray(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::idArray" << endl);
+    if (!m_active) {
+        // See comment in seekId()
+        LOGERR("OHPlaylist::idArray: not active" << endl);
+        return UPNP_E_INTERNAL_ERROR;
+    }
     string idarray;
     int token;
     if (iidArray(idarray, &token)) {
@@ -676,6 +752,7 @@ bool OHPlaylist::iidArray(string& idarray, int *token)
 
 bool OHPlaylist::urlMap(unordered_map<int, string>& umap)
 {
+    //LOGDEB1("OHPlaylist::urlMap\n");
     string sarray; 
     if (iidArray(sarray, 0)) {
         vector<int> ids;
