@@ -43,9 +43,8 @@ static const string sTpProduct("urn:av-openhome-org:service:Receiver:1");
 static const string sIdProduct("urn:av-openhome-org:serviceId:Receiver");
 
 OHReceiver::OHReceiver(UpMpd *dev, const OHReceiverParams& parms)
-    : UpnpService(sTpProduct, sIdProduct, dev), m_dev(dev), 
-      m_pl(parms.pl), m_pr(parms.pr), m_httpport(parms.httpport),
-      m_pm(parms.pm)
+    : OHService(sTpProduct, sIdProduct, dev), m_active(false),
+      m_httpport(parms.httpport), m_sc2mpdpath(parms.sc2mpdpath), m_pm(parms.pm)
 {
     dev->addActionMapping(this, "Play", 
                           bind(&OHReceiver::play, this, _1, _2));
@@ -81,7 +80,8 @@ bool OHReceiver::makestate(unordered_map<string, string> &st)
         if (m_cmd) {
             int status;
             if (m_cmd->maybereap(&status)) {
-                LOGDEB("OHReceiver: sc2cmd exited with status " << status << endl);
+                LOGDEB("OHReceiver: sc2cmd exited with status " << status
+                       << endl);
                 m_cmd = shared_ptr<ExecCmd>(new ExecCmd());
             }
         }
@@ -102,59 +102,36 @@ bool OHReceiver::makestate(unordered_map<string, string> &st)
     return true;
 }
 
-bool OHReceiver::getEventData(bool all, std::vector<std::string>& names, 
-                              std::vector<std::string>& values)
-{
-    //LOGDEB("OHReceiver::getEventData" << endl);
-
-    unordered_map<string, string> state;
-
-    makestate(state);
-
-    unordered_map<string, string> changed;
-    if (all) {
-        changed = state;
-    } else {
-        changed = diffmaps(m_state, state);
-    }
-    m_state = state;
-
-    for (auto it = changed.begin(); it != changed.end(); it++) {
-        //LOGDEB("OHReceiver::getEventData: changed: " << it->first <<
-        // " = " << it->second << endl);
-        names.push_back(it->first);
-        values.push_back(it->second);
-    }
-
-    return true;
-}
-
 void OHReceiver::maybeWakeUp(bool ok)
 {
     if (ok && m_dev)
         m_dev->loopWakeup();
 }
 
-int OHReceiver::play(const SoapIncoming& sc, SoapOutgoing& data)
+void OHReceiver::setActive(bool onoff)
+{
+    m_active = onoff;
+    if (!m_active)
+        iStop();
+}
+
+bool OHReceiver::iPlay()
 {
-    LOGDEB("OHReceiver::play" << endl);
     bool ok = false;
 
-    if (!m_pl) {
+    if (!m_dev->m_ohpl) {
         LOGERR("OHReceiver::play: no playlist service" << endl);
-        return UPNP_E_INTERNAL_ERROR;
+        return false;
     }
     if (m_uri.empty()) {
         LOGERR("OHReceiver::play: no uri" << endl);
-        return UPNP_E_INTERNAL_ERROR;
+        return false;
     }
     if (m_metadata.empty()) {
         LOGERR("OHReceiver::play: no metadata" << endl);
-        return UPNP_E_INTERNAL_ERROR;
+        return false;
     }
 
-    m_dev->m_mpdcli->stop();
-
     int id = -1;
     unordered_map<int, string> urlmap;
     string line;
@@ -176,10 +153,10 @@ int OHReceiver::play(const SoapIncoming& sc, SoapOutgoing& data)
         args.push_back(g_configfilename);
     }
         
-    LOGDEB("OHReceiver::play: executing " << g_sc2mpd_path << endl);
-    ok = m_cmd->startExec(g_sc2mpd_path, args, false, true) >= 0;
+    LOGDEB("OHReceiver::play: executing " << m_sc2mpdpath << endl);
+    ok = m_cmd->startExec(m_sc2mpdpath, args, false, true) >= 0;
     if (!ok) {
-        LOGERR("OHReceiver::play: executing " << g_sc2mpd_path << " failed" 
+        LOGERR("OHReceiver::play: executing " << m_sc2mpdpath << " failed" 
                << endl);
         goto out;
     } else {
@@ -187,17 +164,20 @@ int OHReceiver::play(const SoapIncoming& sc, SoapOutgoing& data)
     }
 
     if (m_pm == OHReceiverParams::OHRP_MPD) {
+        m_dev->m_mpdcli->stop();
+
         // Wait for sc2mpd to signal ready, then play.
         // sc2mpd writes a single line to stdout "CONNECTED" when
         // it gets there, which should be more or less instantaneous
         int timeo = 15;
         if (m_cmd->getline(line, timeo) < 0) {
-            LOGERR("OHReceiver: mpd mode: sc2mpd still not ready to play after " << timeo << " seconds\n");
+            LOGERR("OHReceiver: mpd mode: sc2mpd still not ready to play after "
+                   << timeo << " seconds\n");
             goto out;
         }
         LOGDEB("OHReceiver: sc2mpd sent: " << line);
         // And insert the appropriate uri in the mpd playlist
-        if (!m_pl->urlMap(urlmap)) {
+        if (!m_dev->m_ohpl->urlMap(urlmap)) {
             LOGERR("OHReceiver::play: urlMap() failed" <<endl);
             goto out;
         }
@@ -207,10 +187,17 @@ int OHReceiver::play(const SoapIncoming& sc, SoapOutgoing& data)
             }
         }
         if (id == -1) {
-            ok = m_pl->insertUri(0, m_httpuri, SoapHelp::xmlUnquote(m_metadata),
-                                 &id);
-            if (!ok) {
-                LOGERR("OHReceiver::play: insertUri() failed\n");
+            UpSong metaformpd;
+            string metadata(SoapHelp::xmlUnquote(m_metadata));
+            if (!uMetaToUpSong(metadata, &metaformpd)) {
+                LOGERR("OHReceiver::play: failed to parse metadata " << " Uri [" 
+                       << m_httpuri << "] Metadata [" << metadata << "]"
+                       << endl);
+                goto out;
+            }
+            id = m_dev->m_mpdcli->insertAfterId(m_httpuri, 0, metaformpd);
+            if (id == -1) {
+                LOGERR("OHReceiver::play: insertAfterId() failed\n");
                 goto out;
             }
         }
@@ -226,8 +213,16 @@ out:
     if (!ok) {
         iStop();
     }
+    return ok;
+}
+
+int OHReceiver::play(const SoapIncoming& sc, SoapOutgoing& data)
+{
+    LOGDEB("OHReceiver::play" << endl);
+    if (!m_active && m_dev->m_ohpr)
+        m_dev->m_ohpr->iSetSourceIndexByName("Receiver");
+    bool ok = iPlay();
     maybeWakeUp(ok);
-    LOGDEB("OHReceiver::play: returning\n");
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
@@ -236,39 +231,58 @@ bool OHReceiver::iStop()
     LOGDEB("OHReceiver::iStop()\n");
     if (m_cmd) {
         m_cmd->zapChild();
-        m_cmd = shared_ptr<ExecCmd>(0);
+        m_cmd = shared_ptr<ExecCmd>();
     }
-    m_dev->m_mpdcli->stop();
 
-    unordered_map<int, string> urlmap;
-    // Remove our bogus URi from the playlist
-    if (!m_pl->urlMap(urlmap)) {
-        LOGERR("OHReceiver::stop: urlMap() failed" <<endl);
-        return false;
-    }
-    for (auto it = urlmap.begin(); it != urlmap.end(); it++) {
-        if (it->second == m_httpuri) {
-            m_dev->m_mpdcli->deleteId(it->first);
+    if (m_pm == OHReceiverParams::OHRP_MPD) {
+        m_dev->m_mpdcli->stop();
+        unordered_map<int, string> urlmap;
+        // Remove our bogus URi from the playlist
+        if (!m_dev->m_ohpl->urlMap(urlmap)) {
+            LOGERR("OHReceiver::stop: urlMap() failed" <<endl);
+        }
+        for (auto it = urlmap.begin(); it != urlmap.end(); it++) {
+            if (it->second == m_httpuri) {
+                m_dev->m_mpdcli->deleteId(it->first);
+            }
         }
     }
-    // At least the songcast windows driver never resets the source
-    // index (it does call stop when it deconnects).
-    // I guess that there is no reason to reset the source, and
-    // another CP could just set it to what it wants, but Bubble at
-    // least won't do a thing with the renderer as long as the source
-    // is set to receiver.
-    if (m_pr)
-        m_pr->iSetSourceIndex(0);
-
+    
     return true;
 }
 
 int OHReceiver::stop(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHReceiver::stop" << endl);
-    bool ok = iStop();
-    maybeWakeUp(ok);
-    return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
+    iStop();
+
+    // At least the songcast windows driver never resets the source
+    // index (it does call stop when it deconnects).
+    // I guess that there is no reason to reset the source, and
+    // another CP could just set it to what it wants, but Bubble at
+    // least won't do a thing with the renderer as long as the source
+    // is set to receiver.
+    if (m_dev->m_ohpr)
+        m_dev->m_ohpr->iSetSourceIndexByName("Playlist");
+
+    maybeWakeUp(true);
+    return UPNP_E_SUCCESS;
+}
+
+bool OHReceiver::iSetSender(const string& uri, const string& metadata)
+{
+    // Only do something if data changes, and then first stop any
+    // current playing. We probably should not receive this if we're
+    // not in the stopped state, but just in case...
+    if (m_uri.compare(uri) || m_metadata.compare(metadata)) {
+        if (m_cmd)
+            iStop();
+        m_uri = uri;
+        m_metadata = metadata;
+        LOGDEB("OHReceiver::setSender: uri [" << m_uri << "] meta [" << 
+               m_metadata << "]" << endl);
+    }
+    return true;
 }
 
 int OHReceiver::setSender(const SoapIncoming& sc, SoapOutgoing& data)
@@ -276,17 +290,8 @@ int OHReceiver::setSender(const SoapIncoming& sc, SoapOutgoing& data)
     LOGDEB("OHReceiver::setSender" << endl);
     string uri, metadata;
     bool ok = sc.get("Uri", &uri) && sc.get("Metadata", &metadata);
-
-    // Only do something if data changes, and then first stop any
-    // current playing. We probably should not receive this if we're
-    // not in the stopped state, but just in case...
-    if (ok && (m_uri.compare(uri) || m_metadata.compare(metadata))) {
-        if (m_cmd)
-            iStop();
-        m_uri = uri;
-        m_metadata = metadata;
-        LOGDEB("OHReceiver::setSender: uri [" << m_uri << "] meta [" << 
-               m_metadata << "]" << endl);
+    if (ok) {
+        ok = iSetSender(uri, metadata);
     }
 
     maybeWakeUp(ok);
