@@ -17,21 +17,21 @@
 
 #include "ohvolume.hxx"
 
-#include <upnp/upnp.h>                  // for UPNP_E_SUCCESS, etc
+#include <upnp/upnp.h>
 
-#include <functional>                   // for _Bind, bind, _1, _2
-#include <iostream>                     // for endl
-#include <string>                       // for string
-#include <unordered_map>                // for unordered_map, etc
-#include <utility>                      // for pair
-#include <vector>                       // for vector
+#include <functional>
+#include <iostream>
+#include <string>
+#include <unordered_map>
+#include <utility>
+#include <vector>
 
-#include "libupnpp/log.hxx"             // for LOGDEB
-#include "libupnpp/soaphelp.hxx"        // for SoapOutgoing, SoapIncoming, i2s
+#include "libupnpp/log.hxx"
+#include "libupnpp/soaphelp.hxx"
 
-#include "upmpd.hxx"                    // for UpMpd
-#include "upmpdutils.hxx"               // for diffmaps
-#include "renderctl.hxx"                // for UpMpdRenderCtl
+#include "upmpd.hxx"
+#include "upmpdutils.hxx"
+#include "renderctl.hxx"
 
 using namespace std;
 using namespace std::placeholders;
@@ -41,8 +41,8 @@ static const string millidbperstep("500");
 static const string sTpProduct("urn:av-openhome-org:service:Volume:1");
 static const string sIdProduct("urn:av-openhome-org:serviceId:Volume");
 
-OHVolume::OHVolume(UpMpd *dev, UpMpdRenderCtl *ctl)
-    : UpnpService(sTpProduct, sIdProduct, dev), m_dev(dev), m_ctl(ctl)
+OHVolume::OHVolume(UpMpd *dev)
+    : OHService(sTpProduct, sIdProduct, dev)
 {
     dev->addActionMapping(this,"Characteristics", 
                           bind(&OHVolume::characteristics, this, _1, _2));
@@ -60,6 +60,23 @@ OHVolume::OHVolume(UpMpd *dev, UpMpdRenderCtl *ctl)
                           bind(&OHVolume::mute, this, _1, _2));
     dev->addActionMapping(this,"SetMute", 
                           bind(&OHVolume::setMute, this, _1, _2));
+    dev->addActionMapping(this,"SetBalance", 
+                          bind(&OHVolume::setBalance, this, _1, _2));
+    dev->addActionMapping(this,"Balance", 
+                          bind(&OHVolume::balance, this, _1, _2));
+    dev->addActionMapping(this,"BalanceInc", 
+                          bind(&OHVolume::balanceInc, this, _1, _2));
+    dev->addActionMapping(this,"BalanceDec", 
+                          bind(&OHVolume::balanceDec, this, _1, _2));
+    dev->addActionMapping(this,"SetFade", 
+                          bind(&OHVolume::setFade, this, _1, _2));
+    dev->addActionMapping(this,"Fade", 
+                          bind(&OHVolume::fade, this, _1, _2));
+    dev->addActionMapping(this,"FadeInc", 
+                          bind(&OHVolume::fadeInc, this, _1, _2));
+    dev->addActionMapping(this,"FadeDec", 
+                          bind(&OHVolume::fadeDec, this, _1, _2));
+
 }
 
 bool OHVolume::makestate(unordered_map<string, string> &st)
@@ -70,37 +87,17 @@ bool OHVolume::makestate(unordered_map<string, string> &st)
     st["VolumeLimit"] = "100";
     st["VolumeUnity"] = "100";
     st["VolumeSteps"] = "100";
-    st["VolumeMilliDbPerSteps"] = millidbperstep;
+    st["VolumeMilliDbPerStep"] = millidbperstep;
+    st["Balance"] = "0";
     st["BalanceMax"] = "0";
+    st["Fade"] = "0";
     st["FadeMax"] = "0";
-    int volume = m_ctl->getvolume_i();
+    int volume = m_dev->m_rdctl->getvolume_i();
     st["Volume"] = SoapHelp::i2s(volume);
     st["Mute"] = volume == 0 ? "1" : "0";
     return true;
 }
 
-bool OHVolume::getEventData(bool all, std::vector<std::string>& names, 
-                            std::vector<std::string>& values)
-{
-    //LOGDEB("OHVolume::getEventData" << endl);
-
-    unordered_map<string, string> state, changed;
-    makestate(state);
-    if (all) {
-        changed = state;
-    } else {
-        changed = diffmaps(m_state, state);
-    }
-    m_state = state;
-
-    for (auto it = changed.begin(); it != changed.end(); it++) {
-        names.push_back(it->first);
-        values.push_back(it->second);
-    }
-
-    return true;
-}
-
 int OHVolume::characteristics(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHVolume::charact." << endl);
@@ -120,7 +117,7 @@ int OHVolume::setVolume(const SoapIncoming& sc, SoapOutgoing& data)
     if (!sc.get("Value", &volume)) {
         return UPNP_E_INVALID_PARAM;
     }
-    m_ctl->setvolume_i(volume);
+    m_dev->m_rdctl->setvolume_i(volume);
     m_dev->loopWakeup();
     return UPNP_E_SUCCESS;
 }
@@ -132,17 +129,17 @@ int OHVolume::setMute(const SoapIncoming& sc, SoapOutgoing& data)
     if (!sc.get("Value", &mute)) {
         return UPNP_E_INVALID_PARAM;
     }
-    m_ctl->setmute_i(mute);
+    m_dev->m_rdctl->setmute_i(mute);
     return UPNP_E_SUCCESS;
 }
 
 int OHVolume::volumeInc(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHVolume::volumeInc" << endl);
-    int newvol = m_ctl->getvolume_i() + 1;
+    int newvol = m_dev->m_rdctl->getvolume_i() + 1;
     if (newvol > 100)
         newvol = 100;
-    m_ctl->setvolume_i(newvol);
+    m_dev->m_rdctl->setvolume_i(newvol);
     m_dev->loopWakeup();
     return UPNP_E_SUCCESS;
 }
@@ -150,10 +147,10 @@ int OHVolume::volumeInc(const SoapIncoming& sc, SoapOutgoing& data)
 int OHVolume::volumeDec(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHVolume::volumeDec" << endl);
-    int newvol = m_ctl->getvolume_i() - 1;
+    int newvol = m_dev->m_rdctl->getvolume_i() - 1;
     if (newvol < 0)
         newvol = 0;
-    m_ctl->setvolume_i(newvol);
+    m_dev->m_rdctl->setvolume_i(newvol);
     m_dev->loopWakeup();
     return UPNP_E_SUCCESS;
 }
@@ -161,14 +158,14 @@ int OHVolume::volumeDec(const SoapIncoming& sc, SoapOutgoing& data)
 int OHVolume::volume(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHVolume::volume" << endl);
-    data.addarg("Value", SoapHelp::i2s(m_ctl->getvolume_i()));
+    data.addarg("Value", SoapHelp::i2s(m_dev->m_rdctl->getvolume_i()));
     return UPNP_E_SUCCESS;
 }
 
 int OHVolume::mute(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHVolume::mute" << endl);
-    bool mute = m_ctl->getvolume_i() == 0;
+    bool mute = m_dev->m_rdctl->getvolume_i() == 0;
     data.addarg("Value", mute ? "1" : "0");
     return UPNP_E_SUCCESS;
 }
@@ -179,3 +176,62 @@ int OHVolume::volumeLimit(const SoapIncoming& sc, SoapOutgoing& data)
     data.addarg("Value", "100");
     return UPNP_E_SUCCESS;
 }
+
+int OHVolume::balance(const SoapIncoming& sc, SoapOutgoing& data)
+{
+    LOGDEB("OHVolume::balance" << endl);
+    data.addarg("Value", SoapHelp::i2s(0));
+    return UPNP_E_SUCCESS;
+}
+
+int OHVolume::setBalance(const SoapIncoming& sc, SoapOutgoing& data)
+{
+    LOGDEB("OHVolume::setBalance" << endl);
+    int balance;
+    if (!sc.get("Value", &balance) || balance != 0) {
+        return UPNP_E_INVALID_PARAM;
+    }
+    return UPNP_E_SUCCESS;
+}
+
+int OHVolume::balanceInc(const SoapIncoming& sc, SoapOutgoing& data)
+{
+    LOGDEB("OHVolume::balanceInc" << endl);
+    return UPNP_E_SUCCESS;
+}
+
+int OHVolume::balanceDec(const SoapIncoming& sc, SoapOutgoing& data)
+{
+    LOGDEB("OHVolume::balanceDec" << endl);
+    return UPNP_E_SUCCESS;
+}
+
+
+int OHVolume::fade(const SoapIncoming& sc, SoapOutgoing& data)
+{
+    LOGDEB("OHVolume::fade" << endl);
+    data.addarg("Value", SoapHelp::i2s(0));
+    return UPNP_E_SUCCESS;
+}
+
+int OHVolume::setFade(const SoapIncoming& sc, SoapOutgoing& data)
+{
+    LOGDEB("OHVolume::setFade" << endl);
+    int fade;
+    if (!sc.get("Value", &fade) || fade != 0) {
+        return UPNP_E_INVALID_PARAM;
+    }
+    return UPNP_E_SUCCESS;
+}
+
+int OHVolume::fadeInc(const SoapIncoming& sc, SoapOutgoing& data)
+{
+    LOGDEB("OHVolume::fadeInc" << endl);
+    return UPNP_E_SUCCESS;
+}
+
+int OHVolume::fadeDec(const SoapIncoming& sc, SoapOutgoing& data)
+{
+    LOGDEB("OHVolume::fadeDec" << endl);
+    return UPNP_E_SUCCESS;
+}
